hard_disk_bios segment at 0c800h   ;Western Digital WX2 ROM is here, others?
org 3
hard_disk_init label far
hard_disk_bios ends

power_on_reset segment at 0ffffh
org 0
reset label far
power_on_reset ends

boot_locn segment at 0
org 7c00h
boot_strap label far
boot_locn ends


video_segment equ 0b800h	;0b800h for cga, 0b000h for mono

disk_number equ 0		;fixed disk number, 0 for 1st disk ...

cseg segment
assume cs:cseg

xor ax,ax
mov ds,ax
cmp word ptr ds:[413h],64	;BIOS memory size compare to 64K dip setting
je first_time
jmp reset			;power on reset

first_time:
cld
mov ax,2			;set screen mode 80x25 BW
int 10h

cli
xor di,di
mov si,offset msg0+7c00h
call print_message

mov di,320
mov si,offset msg1+7c00h
call print_message

mov dx,64*64	;segment at 64K
mov bx,64

loop:				;memory sense-initialize loop
call printk
mov es,dx
mov ds,dx
xor ax,ax
mov ds:[0],ax
mov ax,ds:[0]
test ax,ax
jnz done			;stop if memory not present or fail

mov di,ax
mov cx,32768
rep stosw			;initialize 64K

add dh,16
add bx,64
cmp bx,704			;stop at 704K
jb loop

done:
call printk
xor ax,ax
mov ds,ax
mov cl,6
shr dx,cl
mov ds:[413h],dx		;set BIOS memory size

mov di,480
mov si,offset msg2+7c00h
call print_message

call hard_disk_init
cli
in al,21h			;re-enable timer
and al,0feh
out 21h,al

mov di,640
mov si,offset msg3+7c00h
call print_message


relocate:
mov si,7c00h			;relocate code to load HD boot record here
mov di,7c00h+512
xor ax,ax
mov ds,ax
mov es,ax
cld
mov cx,256
rep movsw
jmp hd_boot+512


hd_boot:
xor ax,ax			;reset diskette
mov dx,ax
int 13h

xor ax,ax			;reset fixed disk
mov dx,80h+disk_number
int 13h
jc boot_error

mov ax,201h			;lood boot record
xor bx,bx
mov es,bx
mov ds,bx
mov bx,7c00h
mov dx,80h+disk_number
mov cx,1
int 13h
jc boot_error

mov ax,ds:[7c00h+510]		;test for valid boot record
cmp ax,0aa55h	
jnz boot_error

jmp boot_strap			;GO

boot_error:
mov di,960
mov si,offset err_msg+7c00h
call print_message
sti
here:
jmp short here


printk proc near

mov ax,bx
mov cl,100
div cl		;al = k/100, ah = k % 100
mov cl,al	;cl = k/100, ah = k % 100
mov al,ah	;cl = k/100, al = k % 100
aam		;cl = k/100, ah = (k%100)/10, al = k%10
or ax,3030h
mov bp,video_segment
mov ds,bp
mov ds:[320+46],al
mov al,cl
add al,'0'
mov ds:[320+42],al
mov al,ah
mov ds:[320+44],al
mov al,'K'
mov ds:[320+48],al

ret
printk endp


print_message proc near

mov ax,video_segment
mov es,ax

pm1:
lods byte ptr cs:[si]
test al,al
jz end_pm
stosb
inc di
jmp short pm1

end_pm:
ret
print_message endp


msg0 db 'Fast Hard Disk Boot V3.0',0
msg1 db 'Initializing Memory:',0
msg2 db 'Enabling Hard Disk...',0
msg3 db 'Rebooting from Hard Disk',0
err_msg db 'Unable to Boot from Hard Disk',0

cseg ends
end
